@@ -26,7 +26,6 @@ from .fileutil import BytesForHardlink
 from .log import configure_cli_logging
 from .main import DATABASE_VERSION
 from .main import Fatal
-from .main import check_python_version
 from .main import fatal
 from .main import init_default_indexes
 from .main import set_state_version
@@ -73,7 +72,6 @@ def export(pluginmanager=None, argv=None):
     else:
         # for tests
         argv = [str(x) for x in argv]
-    check_python_version()
     if pluginmanager is None:
         pluginmanager = get_pluginmanager()
     try:
@@ -131,7 +129,6 @@ def import_(pluginmanager=None, argv=None):
     else:
         # for tests
         argv = [str(x) for x in argv]
-    check_python_version()
     if pluginmanager is None:
         pluginmanager = get_pluginmanager()
     try:
@@ -146,16 +143,13 @@ def import_(pluginmanager=None, argv=None):
         add_hard_links_option(parser, pluginmanager)
         parser.add_argument("directory")
         config = parseoptions(pluginmanager, argv, parser=parser)
-        # BBB set import_ flag on args until the option is removed and the
-        # code adjusted
-        config.args.import_ = True
         configure_cli_logging(config.args)
         if config.path_nodeinfo.exists():
             fatal("The path '%s' already contains devpi-server data." % config.serverdir)
         sdir = config.serverdir
         if not (sdir.exists() and len(sdir.listdir()) >= 2):
             set_state_version(config, DATABASE_VERSION)
-        xom = xom_from_config(config)
+        xom = xom_from_config(config, init=True)
         if config.args.wait_for_events:
             xom.thread_pool.start_one(xom.keyfs.notifier)
         init_default_indexes(xom)
@@ -243,7 +237,7 @@ class IndexDump:
 
     def should_dump(self):
         if self.stage.ixconfig["type"] == "mirror":
-            if not self.exporter.config.args.include_mirrored_files:
+            if not self.exporter.config.include_mirrored_files:
                 return False
         return True
 
@@ -331,7 +325,7 @@ class Importer:
         self.filestore = xom.filestore
         self.tw = tw
         self.index_customizers = get_stage_customizer_classes(self.xom)
-        self.types_to_skip = set(self.xom.config.args.skip_import_type or [])
+        self.types_to_skip = set(self.xom.config.skip_import_type or [])
 
     def read_json(self, path):
         self.tw.line("reading json: %s" %(path,))
@@ -425,6 +419,8 @@ class Importer:
                     user = self.xom.model.get_user(username)
                 if user is None:
                     user = self.xom.model.create_user(username, password="")
+                    # missing creation time is set to epoch for regular users
+                    userconfig.setdefault("created", "1970-01-01T00:00:00Z")
                 user._set(userconfig)
 
         # memorize index inheritance structure
@@ -482,7 +478,7 @@ class Importer:
                     stage = self.xom.model.getstage(stagename)
                     if stage is not None:
                         stage.modify(**indexconfig)
-                    elif self.xom.config.args.no_root_pypi:
+                    elif self.xom.config.no_root_pypi:
                         continue
                 if stage is None:
                     stage = user.create_stage(index, **indexconfig)
@@ -551,7 +547,7 @@ class Importer:
         p = self.import_rootdir.join(rel)
         assert p.check(), p
         data = p.read("rb")
-        if self.xom.config.args.hard_links:
+        if self.xom.config.hard_links:
             # wrap the data for additional attribute
             data = BytesForHardlink(data)
             data.devpi_srcpath = p.strpath
@@ -575,15 +571,18 @@ class Importer:
                 entry = self.xom.filestore.maplink(
                     url, stage.username, stage.index, project)
                 entry.file_set_content(data, mapping["last_modified"])
-                (_, links_with_require_python, serial) = stage._load_cache_links(project)
-                if links_with_require_python is None:
-                    links_with_require_python = []
+                (_, links_with_data, serial) = stage._load_cache_links(project)
+                if links_with_data is None:
+                    links_with_data = []
                 links = [(url.basename, entry.relpath)]
                 requires_python = [versions[version].get('requires_python')]
-                for key, href, require_python in links_with_require_python:
+                yanked = [versions[version].get('yanked')]
+                for key, href, require_python, is_yanked in links_with_data:
                     links.append((key, href))
                     requires_python.append(require_python)
-                stage._save_cache_links(project, links, requires_python, serial)
+                    yanked.append(is_yanked)
+                stage._save_cache_links(
+                    project, links, requires_python, yanked, serial)
             # devpi-server-2.1 exported with md5 checksums
             if "md5" in mapping:
                 assert "hash_spec" not in mapping
