@@ -14,6 +14,7 @@ from . import mythread
 from .fileutil import loads
 from .interfaces import IStorageConnection2
 from .log import threadlog, thread_push_log, thread_pop_log
+from .log import thread_change_log_prefix
 from .readonly import get_mutable_deepcopy, ensure_deeply_readonly, \
                       is_deeply_readonly
 from .filestore import FileEntry
@@ -312,7 +313,10 @@ class KeyFS(object):
         return self.get_current_serial() + 1
 
     def get_current_serial(self):
-        with self.get_connection() as conn:
+        tx = getattr(self._threadlocal, "tx", None)
+        if tx is not None:
+            return tx.conn.last_changelog_serial
+        with self.get_connection(write=False) as conn:
             return conn.last_changelog_serial
 
     def get_last_commit_timestamp(self):
@@ -345,18 +349,23 @@ class KeyFS(object):
             key = key(**key.extract_params(relpath))
         return key
 
+    def _tx_prefix(self):
+        tx = self._threadlocal.tx
+        return "[%stx%s]" % ("W" if tx.write else "R", tx.at_serial)
+
     def begin_transaction_in_thread(self, write=False, at_serial=None):
         if write and self._readonly:
             raise self.ReadOnly()
         assert not hasattr(self._threadlocal, "tx")
         tx = Transaction(self, write=write, at_serial=at_serial)
         self._threadlocal.tx = tx
-        thread_push_log("[%stx%s]" %("W" if write else "R", tx.at_serial))
+        thread_push_log(self._tx_prefix())
         return tx
 
     def clear_transaction(self):
-        thread_pop_log()
+        prefix = self._tx_prefix()
         del self._threadlocal.tx
+        thread_pop_log(prefix)
 
     def restart_as_write_transaction(self):
         if self._readonly:
@@ -364,17 +373,17 @@ class KeyFS(object):
         tx = self.tx
         if tx.write:
             raise RuntimeError("Can't restart a write transaction.")
-        thread_pop_log()
+        old_prefix = self._tx_prefix()
         tx.restart(write=True)
-        thread_push_log("[Wtx%s]" %(tx.at_serial))
+        thread_change_log_prefix(self._tx_prefix(), old_prefix)
 
     def restart_read_transaction(self):
         tx = self.tx
         if tx.write:
             raise RuntimeError("Can only restart a read transaction.")
-        thread_pop_log()
+        old_prefix = self._tx_prefix()
         tx.restart(write=False)
-        thread_push_log("[Rtx%s]" %(tx.at_serial))
+        thread_change_log_prefix(self._tx_prefix(), old_prefix)
 
     def rollback_transaction_in_thread(self):
         try:
@@ -400,6 +409,7 @@ class KeyFS(object):
 
 
 class PTypedKey:
+    __slots__ = ('keyfs', 'name', 'pattern', 'rex_reverse', 'type')
     rex_braces = re.compile(r'\{(.+?)\}')
 
     def __init__(self, keyfs, key, type, name):
@@ -443,6 +453,8 @@ class KeyChangeEvent:
 
 
 class TypedKey:
+    __slots__ = ('keyfs', 'name', 'params', 'relpath', 'type')
+
     def __init__(self, keyfs, relpath, type, name, params=None):
         self.keyfs = keyfs
         self.relpath = relpath
@@ -450,13 +462,6 @@ class TypedKey:
         self.name = name
         self.params = params or {}
 
-    @cached_property
-    def params(self):
-        key = self.keyfs.get_key(self.name)
-        if isinstance(key, PTypedKey):
-            return key.extract_params(self.relpath)
-        return {}
-
     def __hash__(self):
         return hash(self.relpath)
 
