@@ -1,7 +1,6 @@
 from __future__ import unicode_literals
 import getpass
 import posixpath
-import sys
 import py
 import re
 import json
@@ -24,7 +23,7 @@ from time import gmtime, strftime
 from .auth import hash_password, verify_and_update_password_hash
 from .config import hookimpl
 from .filestore import FileEntry
-from .log import threadlog, thread_current_log
+from .log import threadlog
 from .readonly import get_mutable_deepcopy
 
 
@@ -49,7 +48,7 @@ def apply_filter_iter(items, filter_iter):
 
 def run_passwd(root, username):
     user = root.get_user(username)
-    log = thread_current_log()
+    log = threadlog
     if user is None:
         log.error("user %r not found" % username)
         return 1
@@ -1411,15 +1410,15 @@ def devpiserver_get_stage_customizer_classes():
 
 class ELink(object):
     """ model Link using entrypathes for referencing. """
+    __slots__ = ('_entry', 'basename', 'filestore', 'linkdict', 'project', 'version')
+
     def __init__(self, filestore, linkdict, project, version):
+        self._entry = notset
         self.filestore = filestore
         self.linkdict = linkdict
         self.basename = posixpath.basename(self.entrypath)
         self.project = project
         self.version = version
-        if sys.version_info < (3,0):
-            for key in linkdict:
-                assert py.builtin._istext(key)
 
     @property
     def relpath(self):
@@ -1454,15 +1453,14 @@ class ELink(object):
     def __repr__(self):
         return "<ELink rel=%r entrypath=%r>" % (self.rel, self.entrypath)
 
-    @cached_property
+    @property
     def entry(self):
-        return self.filestore.get_file_entry(self.entrypath)
+        if self._entry is notset:
+            self._entry = self.filestore.get_file_entry(self.entrypath)
+        return self._entry
 
     def add_log(self, what, who, **kw):
         d = {"what": what, "who": who, "when": gmtime()[:6]}
-        if sys.version_info < (3,0):
-            # make sure keys are unicode as they are on py3
-            kw = dict((py.builtin.text(name), value) for name, value in kw.items())
         d.update(kw)
         self._log.append(d)
 
